@@ -9,21 +9,25 @@ from django.utils.translation import ugettext as _
 from django.core.exceptions import ValidationError
 from django.forms.models import model_to_dict
 from django.contrib.sites.models import Site
+from django.core.urlresolvers import reverse
 from django.utils.timezone import now
 from django.contrib import messages
 from django.db import models
 from django.conf import settings
 
+from ipaddress import IPv4Network, IPv4Address
 from httpagentparser import detect
 
 from secureauth.utils.token import random_seed, check_seed, get_google_url
+from secureauth.utils import is_phone, get_formatted_phone
 from secureauth.utils import get_ip, get_geo, inet_aton
-from secureauth.utils.codes import RandomPassword
+from secureauth.utils.codes import RandomPassword, md5
 from secureauth.utils import send_sms, send_mail
+from secureauth.utils import render_template
 from secureauth.utils.sign import Sign
-from secureauth.utils import is_phone
+
 from secureauth.defaults import (
-    SMS_MESSAGE, SMS_CODE_LEN, SMS_AGE, SMS_FROM,
+    SMS_MESSAGE, SMS_CODE_LEN, SMS_ASCII, SMS_AGE, SMS_FROM,
     CODE_RANGES, CODE_LEN, SMS_NOTIFICATION_MESSAGE, SMS_NOTIFICATION_SUBJECT,
     LOGIN_ATTEMPT, BAN_TIME, CHECK_ATTEMPT, CODES_SUBJECT, TOTP_NAME)
 
@@ -51,7 +55,7 @@ class UserAuthAbstract(models.Model):
 
     def update_last_verified(self):
         self.last_verified = now()
-        self.save(update_fields=['last_verified'])
+        self.save() #update_fields=['last_verified']
 
     def check_auth_code(self, auth_code, force_check=False):
         if (self.code and self.enabled) or force_check:
@@ -87,7 +91,7 @@ class UserAuthCode(UserAuthAbstract):
     def get_code_number(self):
         number = random.choice(range(1, CODE_RANGES + 1))
         self.number = Sign().sign(number)
-        self.save(update_fields=['number'])
+        self.save() #update_fields=['number']
         return number
 
     def make(self):
@@ -130,7 +134,8 @@ class UserAuthToken(UserAuthAbstract):
         return get_google_url(Sign().unsign(self.code), TOTP_NAME % data)
 
     def _code_is_valid(self, code):
-        return check_seed(Sign().unsign(self.code), int(code))
+        if code.isdigit():
+            return check_seed(Sign().unsign(self.code), int(code))
 
     def make(self):
         if not self.code:
@@ -150,12 +155,12 @@ class UserAuthPhone(UserAuthAbstract):
                 _('Phone does not contain spaces and must be starts with a +'))
 
     def save(self, *args, **kwargs):
-        if str(self.phone).startswith('+'):
-            self.phone = Sign().sign(self.phone)
+        if self.phone.startswith('+'):
+            self.phone = Sign().sign(get_formatted_phone(self.phone))
         super(UserAuthPhone, self).save(*args, **kwargs)
 
     def send_sms(self):
-        code = str(RandomPassword().get(SMS_CODE_LEN)).lower()
+        code = str(RandomPassword().get(SMS_CODE_LEN, SMS_ASCII)).lower()
         send_sms(SMS_FROM, Sign().unsign(self.phone), SMS_MESSAGE % code)
         self.code = Sign().sign(code)
         self.save()
@@ -213,8 +218,9 @@ class UserAuthActivity(models.Model):
     agent = models.CharField(
         _('Browser'), max_length=255, null=True, blank=True)
     confirm_method = models.CharField(
-        _('Confirm method'), max_length=10, choices=AUTH_TYPES,
+        _('Confirm Method'), max_length=10, choices=AUTH_TYPES,
         null=True, blank=True)
+    notified = models.BooleanField(default=False, editable=False)
 
     class Meta:
         ordering = ('-id',)
@@ -237,12 +243,16 @@ class UserAuthActivity(models.Model):
     @classmethod
     def check_location(cls, request):
         obj = cls.objects.filter(user_id=request.user.pk).order_by('-id')[:1]
-        if obj.exists() and get_geo(get_ip(request)) != obj[0].geo:
-            show_msg = """%s<br/>GEO: %s<br/>IP: %s<br/>PC: %s<br/>%s""" % (
-                unicode(_('Your location has changed. Former location:')),
-                obj[0].geo, obj[0].ip, obj[0].agent, unicode(
-                    _('If it was not you, then change authorization data.')))
-            messages.warning(request, show_msg)
+        if obj.exists():
+            geo = get_geo(get_ip(request))
+            obj = obj[0]
+            if not obj.notified and geo != obj.geo:
+                messages.warning(request, render_template(
+                    'secureauth/location_message.html', {'obj': obj}))
+                UserAuthNotification.notify(
+                    request, _('Your location has changed to %s' % geo))
+                obj.notified = True
+                obj.save()
 
 
 class UserAuthAttempt(models.Model):
@@ -258,7 +268,7 @@ class UserAuthAttempt(models.Model):
     def store(cls, request):
         obj = cls.get_obj(request)
         obj.attempt = models.F('attempt') + 1
-        obj.save(update_fields=['attempt'])
+        obj.save() #update_fields=['attempt']
 
     @classmethod
     def remove(cls, request):
@@ -272,6 +282,7 @@ class UserAuthAttempt(models.Model):
         if obj.attempt > LOGIN_ATTEMPT:
             if (now() - obj.created).seconds < BAN_TIME:
                 return True
+            obj.delete()
 
     @classmethod
     def clean(cls):
@@ -287,3 +298,70 @@ class UserAuthLogging(models.Model):
     @classmethod
     def is_enabled(cls, request):
         return cls.objects.filter(user=request.user, enabled=1).exists()
+
+
+class UserAuthIP(models.Model):
+    user = models.OneToOneField(
+        settings.AUTH_USER_MODEL, verbose_name=_('User'))
+    enabled = models.BooleanField(_('Enabled'), default=False)
+
+    @classmethod
+    def is_enabled(cls, user):
+        return cls.objects.filter(user=user, enabled=1).exists()
+
+
+class UserAuthIPRange(models.Model):
+    ip = models.ForeignKey(UserAuthIP, editable=False)
+    ip_data = models.CharField(
+        _('IP/IP range'), max_length=18,
+        help_text='xxx.xxx.xxx.xxx or xxx.xxx.xxx.xxx/24')
+    is_ip = models.BooleanField(_('Is IP'), default=False)
+
+    class Meta:
+        unique_together = (('ip', 'ip_data', 'is_ip',),)
+
+    @classmethod
+    def is_allowed(cls, request, user):
+        ip = get_ip(request)
+        if cls.objects.filter(ip__user=user, ip_data=ip).exists():
+            return True
+
+        range_list = cls.objects.values_list('ip_data', flat=True).filter(
+            ip__user=user, is_ip=False)
+        user_ip = IPv4Address(unicode(ip))
+
+        if any([user_ip in IPv4Network(ip_range) for ip_range in range_list]):
+            return True
+
+    @classmethod
+    def send_link(cls, request, user):
+        data = {
+            'ip': get_ip(request),
+            'user_agent': md5(request.META.get('HTTP_USER_AGENT')),
+        }
+        link = 'http://%s%s?data=%s' % (
+            Site.objects.get_current(),
+            reverse('auth_login'),
+            Sign().sign(data)
+        )
+        send_mail(
+            [user.email], _('Link for unlock access'), link
+        )
+
+    @classmethod
+    def check_access(cls, request):
+        if request.GET.get('data'):
+            data = Sign().unsign(request.GET.get('data'), SMS_AGE * 2)
+            if data.get('ip') == get_ip(request):
+                user_agent = md5(request.META['HTTP_USER_AGENT'])
+                if data.get('user_agent') == user_agent:
+                    return True
+
+    @classmethod
+    def is_blocked(cls, request, user):
+        if UserAuthIP.is_enabled(user):
+            is_allowed = cls.is_allowed(request, user)
+            access_checked = cls.check_access(request)
+            if not is_allowed and not access_checked:
+                cls.send_link(request, user)
+                return True
